/**
* Copyright (C) 2002 Alex Gitelman
*
*/
#ifdef SIMULATOR
#include <fcntl.h>
#endif
#include <file.h>
#include "ajf.h"
#include <malloc.h>
#include <string.h>
#include <errno.h>

int _font_error_code = 0;
char _font_error_msg[50];

void report_error(int code, const char *msg)
{
    char berr[51];
    _font_error_code = code;

    snprintf(berr, 50, "%d: %s", code, msg);
    //berr[15] = 0;

    strcpy(_font_error_msg, berr);
#ifdef SIMULATOR    

    printf("%s\n",berr);
#else
    panicf("%s\n",berr);

#endif
}


#define BUFLEN 64
#define MAX_FONT_BUFLEN 10000

unsigned char* ajf_read_font(const char* fname)
{
    unsigned char buf[BUFLEN];
    unsigned char *out_buf; //[10000];
    int count;
    unsigned char *font_buf = 0;
    unsigned char *font = 0;
    unsigned char *new_buf;
    int idx = 0;
    int len = 0;
#ifdef SIMULATOR
    int fd = open(fname, O_RDONLY|O_BINARY);
#else
    int fd = open(fname, O_RDONLY); // Always binary on Archos
#endif
    if (fd<0)
    {
        char errb[51];
#ifdef SIMULATOR    
        snprintf(errb, 50, "%d %s. ", _errno(), fname);
#else
        snprintf(errb, 50, "%d %s. ", errno, fname);
#endif
        report_error(100, errb);
        return 0;
    }
    out_buf = malloc(MAX_FONT_BUFLEN);
    memset(buf, 0x00, sizeof(buf));
    memset(out_buf, 0x00, MAX_FONT_BUFLEN);

    count = read(fd, buf, BUFLEN);
    memcpy(&out_buf[idx], buf, count);
    idx+=count;

    while (count>0) //count>0&&count==1000)
    {
        count = read(fd, buf, BUFLEN);
        if (count<=0)
            break;
        memcpy(&out_buf[idx], buf, count);
        idx+=count;
    }
    

    close(fd);
    font_buf = malloc(idx);
    if (font_buf==0)
        panicf("malloc(%d) = 0 in readAJF", idx);
    memcpy(font_buf, out_buf, idx);
    free(out_buf);
    if (font_buf[0]==MAGIC1 && font_buf[1]==MAGIC2)
        return font_buf;

    report_error(300, "Wrong Magic");
    return 0;
}


void ajf_get_charbuf(unsigned char c, unsigned char* font, unsigned char *src, int *w, int *h)
{
    int i;
    unsigned short height = (unsigned short)font[HEIGHT_OFFSET];
    int rows = (height-1)/8 + 1;
    unsigned char *p = &font[CHARS_OFFSET];
    *h = font[HEIGHT_OFFSET];
    for(i=0;i<c;i++)
    {
        p+=(1 + p[0]);
    }
    memcpy(src, &p[0], p[0]);
    *w = (int)p[0]/rows;
    return;
}

void ajf_get_charsize(unsigned char c, unsigned char* font, int *width, int *height)
{
    int i;
    int rows;  
    unsigned char *p = &font[CHARS_OFFSET];
    *height = font[HEIGHT_OFFSET];
    rows = (*height-1)/8 + 1;
    for(i=0;i<c;i++)
    {
        p+=(1 + p[0]);
    }
    *width = (int)p[0]/rows;
    return;
}

int ajf_get_fontheight(unsigned char* font)
{
    return (int)font[HEIGHT_OFFSET];
}
